% $Id: FloatComparisons.oz,v 1.4 2006/10/27 08:57:49 leavens Exp $
% Approximate floating point numeric comparison procedures for Oz.
% AUTHOR: Gary T. Leavens

functor $
export
   within: Within
   withinMaker: WithinMaker
   relative: Relative
   relativeMaker: RelativeMaker
   compareLists: CompareLists
   withinLists: WithinLists
   relativeLists: RelativeLists
define
   %% Return true iff the difference between X and Y
   %% is no larger than Epsilon
   fun {Within Epsilon X Y} {Abs X-Y} =< Epsilon end
   
   %% Partly curried version of Within
   fun {WithinMaker Epsilon} fun {$ X Y} {Within Epsilon X Y} end end

   %% Return true iff the corresponding lists are
   %% equal relative to the given predicate
   fun {CompareLists Pred Xs Ys}
      case Xs#Ys of
	 nil#nil then true
      [] (X|Xr)#(Y|Yr) then {Pred X Y} andthen {CompareLists Pred Xr Yr}
      else false
      end
   end

   %% Return true iff the lists are equal
   %% in the sense that the corresponging elements
   %% are equal to within Epsilon
   fun {WithinLists Epsilon Xs Ys}
      {CompareLists {WithinMaker Epsilon} Xs Ys}
   end

   %% Return true iff the ratio of X-Y to Y is within Epsilon
   fun {Relative Epsilon X Y} {Abs X-Y} =< Epsilon*{Abs Y} end
   
   %% Partly curried version of Relative
   fun {RelativeMaker Epsilon} fun {$ X Y} {Relative Epsilon X Y} end end

   %% Return true iff the lists are equal
   %% in the sense that the corresponging elements
   %% are relatively equal to within Epsilon
   fun {RelativeLists Epsilon Xs Ys}
      {CompareLists {RelativeMaker Epsilon} Xs Ys}
   end
end
